#include "PsqlServer.h"
#include "DialogInput.h"
#include <iostream.h>

PsqlServer::PsqlServer(void)
	:	conn(0), connected(false)
	{
	}

PsqlServer::~PsqlServer(void)
  {
  }

QueryResult PsqlServer::connect(QString host, QString user, QString passwd, int port)
  {
  pg_host=host;
  pg_user=user;
  pg_passwd=passwd;
  pg_port=port;
/*  DialogInput dlg;
  if(!dlg.exec())
  	{
  	QueryResult res;
  	res.type = QueryResult::Error;
  	res.message = "Connection canceled by user";
  	connected = false;
  	return res;
  	}*/
//  QString baseName = dlg.getText();
  QString baseName("template1");
  openedDB="ttemplate1";
  QString portStr;
  portStr.setNum(port);
  const char *pcPort = port==0 ? 0 : (const char *)portStr;
  conn = PQsetdbLogin(host, pcPort, 0, 0, (const char *)baseName, user, passwd);
  QueryResult res;
  if(PQstatus(conn) == CONNECTION_OK)
  	{
  	res.type = QueryResult::Ok;
  	connected = true;
  	return res;
  	}
  else
  	{
  	res.type = QueryResult::Error;
  	res.message = PQerrorMessage(conn);
  	connected = false;
	openedDB="";
  	return res;
  	}
  }

void PsqlServer::close(void)
  {
  PQfinish(conn);
  connected = false;
  }

QueryResult PsqlServer::selectBase(QString baseName)
  {
  if(baseName==openedDB){
      QueryResult res;
      res.type = QueryResult::Ok;
      return res;
  }      

  close();
 
  int port=pg_port;
  QString host=pg_host;
  QString user=pg_user;
  QString passwd=pg_passwd;
  
  QString portStr;
  portStr.setNum(port);
  const char *pcPort = port==0 ? 0 : (const char *)portStr;
  conn = PQsetdbLogin(host, pcPort, 0, 0, (const char *)baseName, user, passwd);
  QueryResult res;
  if(PQstatus(conn) == CONNECTION_OK)
  	{
  	res.type = QueryResult::Ok;
  	connected = true;
	openedDB=baseName;
  	return res;
  	}
  else
  	{
  	res.type = QueryResult::Error;
  	res.message = PQerrorMessage(conn);
  	connected = false;
	openedDB="";
  	return res;
  	}
  }

QueryResult PsqlServer::query(QString query)
  {
  QueryResult queryRes;
	PGresult *res = PQexec(conn, query);
	switch(PQresultStatus(res))
		{
		case PGRES_EMPTY_QUERY:
			queryRes.type = QueryResult::Ok;
			queryRes.message = "Empty query";
			break;
		case PGRES_COMMAND_OK:
			queryRes.type = QueryResult::Ok;
			queryRes.message = "Command OK - ";
			queryRes.message += PQcmdStatus(res);
			break;
		case PGRES_COPY_OUT:
			queryRes.type = QueryResult::Ok;
			queryRes.message = "Copy OUT";
			break;
		case PGRES_COPY_IN:
			queryRes.type = QueryResult::Ok;
			queryRes.message = "Copy IN";
			break;
		case PGRES_BAD_RESPONSE:
			queryRes.type = QueryResult::Error;
			queryRes.message = "Bad response - ";
			queryRes.message += PQerrorMessage(conn);
			break;
		case PGRES_NONFATAL_ERROR:
			queryRes.type = QueryResult::Error;
			queryRes.message = PQerrorMessage(conn);
			break;
		case PGRES_FATAL_ERROR:
			queryRes.type = QueryResult::Error;
			queryRes.message = PQerrorMessage(conn);
			break;
		case PGRES_TUPLES_OK:
			queryRes.type == QueryResult::Set;
			putTuples(queryRes, res);
			break;
		default:
			queryRes.type = QueryResult::Error;
			queryRes.message = "Unknown error";
			break;						
		}	
	PQclear(res);
	return queryRes;
	}
	
void PsqlServer::putTuples(QueryResult &queryRes, PGresult *res)
	{
	queryRes.numColumns = PQnfields(res);
	int numRows = PQntuples(res);
	// Les champs
	queryRes.fields = "";
	for(int i=0; i<queryRes.numColumns; i++)
		{
		queryRes.fields += PQfname(res, i);
		if(i < queryRes.numColumns-1)
			queryRes.fields += '\t';
		}
	// Les tuples
	queryRes.result.clear();
	for(int row=0; row<numRows; row++)
		{
		QString tuple = "";
		for(int	col=0; col<queryRes.numColumns; col++)
			{
			QString value = PQgetvalue(res, row, col);
			if(col < queryRes.numColumns-1)
				value += " \t";
			tuple += value;
			}
		queryRes.result.append(tuple);
		}	
	// Misc
	queryRes.type = QueryResult::Set;
  queryRes.message = "Query OK";
	}

QueryResult PsqlServer::getBases(void)
  {
/*  QueryResult queryRes;
  queryRes.type == QueryResult::Set;
  queryRes.result.append(PQdb(conn));
  return queryRes;
*/
  QueryResult queryRes;
	char *query = "SELECT datname FROM pg_database ORDER BY datname";
	PGresult *result = PQexec(conn, query);	
  queryRes.type = QueryResult::Set;
  int nbTuples = PQntuples(result);
  for(int i=0; i<nbTuples; i++)
  	queryRes.result.append(PQgetvalue(result, i, 0));
  PQclear(result);
  return queryRes;

  }

QueryResult PsqlServer::getTables(QString baseName, QString tableName)
  {
  selectBase(baseName);
  QueryResult queryRes;
	char *query = "SELECT relname FROM pg_class WHERE relkind = 'r' AND relname !~ 'pg_' AND relname !~ '^xin[vx][0-9]+' ORDER BY relname";
	PGresult *result = PQexec(conn, query);	
  queryRes.type = QueryResult::Set;
  int nbTuples = PQntuples(result);
  for(int i=0; i<nbTuples; i++)
  	queryRes.result.append(PQgetvalue(result, i, 0));
  PQclear(result);
  return queryRes;
  }

QueryResult PsqlServer::getColumns(QString baseName, QString tableName)
 	{
	selectBase(baseName);
  QueryResult queryRes;
	QString query("");
//	query.sprintf("select attnum,attname,typname,attlen,attnotnull,atttypmod,usename,usesysid,pg_class.oid,relpages,reltuples,relhaspkey,relhasrules,relacl from pg_class,pg_user,pg_attribute,pg_type where (pg_class.relname='%s') and (pg_class.oid=pg_attribute.attrelid) and (pg_class.relowner=pg_user.usesysid) and (pg_attribute.atttypid=pg_type.oid) and attnum>0 order by attnum",(const char*)tableName);
	query="select attname,typname,attlen,attnotnull,relhaspkey,relhasrules from pg_class,pg_user,pg_attribute,pg_type where (pg_class.relname='";
	query+=tableName;
	query+="') and (pg_class.oid=pg_attribute.attrelid) and (pg_class.relowner=pg_user.usesysid) and (pg_attribute.atttypid=pg_type.oid) and attnum>0 order by attnum";
	PGresult *result = PQexec(conn, (const char*)query);	
  queryRes.type = QueryResult::Set;
  int nbTuples = PQntuples(result);
  for(int i=0; i<nbTuples; i++)
  	queryRes.result.append(PQgetvalue(result, i, 0));
  PQclear(result);
  return queryRes;
/*	QueryResult res;
 	res.type = QueryResult::Unsuported;
 	return res;*/
 	}

QueryResult PsqlServer::shutdown(void)
  {
  QueryResult queryRes;
  queryRes.type = QueryResult::Error;
  queryRes.message = "Sorry, this is not supported by the PostGres plugin ;(";
  return queryRes;
  }

QueryResult PsqlServer::reload(void)
  {
  QueryResult queryRes;
  queryRes.type = QueryResult::Error;
  queryRes.message = "Sorry, this is not supported by the PostGres plugin ;(";
  return queryRes;
  }

QString PsqlServer::serverInfo(void)
  {
  return "Type: PostGres\n";
  }

QueryResult PsqlServer::createBase(QString baseName)
  {
/*  QueryResult queryRes;
  queryRes.type = QueryResult::Error;
  queryRes.message = "Sorry, this is not supported by the PostGres plugin ;(";
  return queryRes;*/
  QString  Query("");
  Query="CREATE DATABASE ";
  Query+=baseName;
  return query(Query);
  }

QueryResult PsqlServer::dropBase(QString baseName)
  {
/*  QueryResult queryRes;
  queryRes.type = QueryResult::Error;
  queryRes.message = "Sorry, this is not supported by the PostGres plugin ;(";
  return queryRes;*/
  QString  Query("");
  Query="DROP DATABASE ";
  Query+=baseName;
  return query(Query);
  }

IObject *PsqlServer::queryInterface(UIID id)
  {
  if(id == UIID_SQLSERVER)
    return this;
  else
    return 0;
  }
